# coding=utf-8
from earthdiagnostics.diagnostic import Diagnostic
from earthdiagnostics.utils import Utils, TempFile
from earthdiagnostics.variable import Domain


class MonthlyPercentile(Diagnostic):
    """
    Calculates the montlhy percentiles

    :param data_manager: data management object
    :type data_manager: DataManager
    :param startdate: startdate
    :type startdate: str
    :param member: member number
    :type member: int
    :param chunk: chunk's number
    :type chunk: int
    :param variable: variable to average
    :type variable: str
    """

    alias = 'monpercent'
    "Diagnostic alias for the configuration file"

    def __init__(self, data_manager, startdate, member, chunk, variable, domain, percentile):
        Diagnostic.__init__(self, data_manager)
        self.startdate = startdate
        self.member = member
        self.chunk = chunk
        self.variable = variable
        self.domain = domain
        self.percentile = percentile

    def __eq__(self, other):
        return self.startdate == other.startdate and self.member == other.member and self.chunk == other.chunk and \
            self.domain == other.domain and self.variable == other.variable and self.percentile == other.percentile

    def __str__(self):
        return 'Monthly percentile {0} Startdate: {0} Member: {1} Chunk: {2} ' \
               'Variable: {3}:{4}'.format(self.startdate, self.member, self.chunk, self.domain, self.variable)

    @classmethod
    def generate_jobs(cls, diags, options):
        """
        Creates a job for each chunk to compute the diagnostic

        :param diags: Diagnostics manager class
        :type diags: Diags
        :param options: domain, variable, percentil number, maximum depth (level)
        :type options: list[str]
        :return:
        """
        num_options = len(options) - 1
        if num_options < 3:
            raise Exception('You must specify the variable (and its domain) to average vertically and '
                            'the percentil you want')
        if num_options > 3:
            raise Exception('You must specify between one and three parameters for the vertical mean')

        domain = Domain(options[1])
        variable = options[2]
        percentile = int(options[3])
        if percentile < 0 or percentile > 100:
            raise Exception('Percentile value must be in the interval [0,100]')

        job_list = list()
        for startdate, member, chunk in diags.config.experiment.get_chunk_list():
            job_list.append(MonthlyPercentile(diags.data_manager, startdate, member, chunk,
                                              variable, domain, percentile))
        return job_list

    def compute(self):
        """
        Runs the diagnostic
        """
        variable_file = self.data_manager.get_file(self.domain, self.variable, self.startdate, self.member, self.chunk)
        temp = TempFile.get()
        Utils.cdo.monpctl(str(self.percentile), input=[variable_file, '-monmin ' + variable_file,
                                                       '-monmax ' + variable_file], output=temp)
        Utils.rename_variable(temp, 'lev', 'ensemble', False, True)
        self.send_file(temp, self.domain, '{0}p{1}'.format(self.variable, self.percentile), self.startdate, self.member,
                       self.chunk, frequency='mon', rename_var=self.variable)



